<?php

namespace Drupal\role_toggle\AuthenticationManager;

use Drupal\Core\Authentication\AuthenticationProviderChallengeInterface;
use Drupal\Core\Authentication\AuthenticationProviderFilterInterface;
use Drupal\Core\Authentication\AuthenticationProviderInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AnonymousUserSession;
use Drupal\role_toggle\AccountDecorator\RoleToggleAccountDecorator;
use Drupal\role_toggle\RoleToggleState;
use Drupal\role_toggle\RoleToggleUrlManager;
use Symfony\Component\HttpFoundation\Request;

class RoleToggleAuthenticationManagerDecorator implements AuthenticationProviderInterface, AuthenticationProviderFilterInterface, AuthenticationProviderChallengeInterface {

  /**
   * @var \Drupal\Core\Authentication\AuthenticationProviderInterface
   */
  protected $decorated;
  /**
   * The role toggle state.
   *
   * @var \Drupal\role_toggle\RoleToggleState
   */
  private $roleToggleState;

  /**
   * @var \Drupal\role_toggle\RoleToggleUrlManager
   */
  private $roleToggleUrlManager;

  /**
   * @var \Drupal\user\RoleStorageInterface
   */
  private $roleStorage;

  /**
   * RoleToggleAuthenticationManagerDecorator constructor.
   *
   * @param \Drupal\Core\Authentication\AuthenticationProviderInterface|\Drupal\Core\Authentication\AuthenticationProviderFilterInterface|\Drupal\Core\Authentication\AuthenticationProviderChallengeInterface $decorated
   * @param \Drupal\role_toggle\RoleToggleState $roleToggleState
   * @param \Drupal\role_toggle\RoleToggleUrlManager $roleToggleUrlManager
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   */
  public function __construct(AuthenticationProviderInterface $decorated, RoleToggleState $roleToggleState, RoleToggleUrlManager $roleToggleUrlManager, EntityTypeManagerInterface $entityTypeManager) {
    $this->decorated = $decorated;
    $this->roleToggleState = $roleToggleState;
    $this->roleToggleUrlManager = $roleToggleUrlManager;
    $this->roleStorage = $entityTypeManager->getStorage('user_role');
  }


  /**
   * {@inheritdoc}
   */
  public function applies(Request $request) {
    return $this->decorated->applies($request);
  }

  /**
   * {@inheritdoc}
   */
  public function authenticate(Request $request) {
    $account = $this->decorated->authenticate($request);
    $this->roleToggleState->init($account);
    if ($account) {
      if ($rolesBag = $this->roleToggleUrlManager->parseToggledRolesBag($request, $account)) {
        $originalAccount = $account;
        // We never grant more permissions than the user originally has.
        $extraPermissions = array_filter($this->getPassThroughPermissions(), function ($permission) use ($account) {
          return $account->hasPermission($permission);
        });
        if ($rolesBag->hasAuthenticated()) {
          // The default Cookie auth provider returns a UserSession object the
          // roles of which can not be changed. So decorate.
          // Loading and saving the user is then managed by EntityHooks.
          $account = new RoleToggleAccountDecorator($account, $this->roleStorage, $rolesBag->idsWithLocked(), $extraPermissions);
          $this->roleToggleState->setAccount($account);
          // Clear the user cache for the toggled user to ensure that all other
          // modules that load the user see the changed roles via EntityHooks.
          // If ever https://www.drupal.org/node/2345611 lands, this is needed.
          // It looks as this is needed already now, as in theory current user
          // should not be accessed before authentication (there is a core
          // comment about this somewhere).
          // In practice, there seem to be early loadings of the current user,
          // e.g. when bundle labels are translated and translations are cached
          // by current user in \Drupal\locale\LocaleLookup::getCid.
          // We do not do this for toggle to anonymous though, as modules do not
          // load the account's user in this case.
          $this->roleStorage->loadUnchanged($originalAccount->id());
        }
        else {
          // The query parameter said we want to be anonymous.
          $account = new RoleToggleAccountDecorator(new AnonymousUserSession(), $this->roleStorage, [AccountInterface::ANONYMOUS_ROLE] , $extraPermissions);
          $this->roleToggleState->setAccount($account);
        }
      }
    }
    return $account;
  }

  /**
   * {@inheritdoc}
   */
  public function challengeException(Request $request, \Exception $previous) {
    return $this->decorated->challengeException($request, $previous);
  }

  /**
   * {@inheritdoc}
   */
  public function appliesToRoutedRequest(Request $request, $authenticated) {
    return $this->decorated->appliesToRoutedRequest($request, $authenticated);
  }

  /**
   * @return array
   */
  protected function getPassThroughPermissions() {
    return [
      'access toolbar',
      'role_toggle regular',
      'role_toggle all',
    ];
  }

}
