<?php

namespace Drupal\role_toggle;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class RoleToggleForm implements FormInterface, ContainerInjectionInterface {

  /**
   * The role toggle state.
   *
   * @var \Drupal\role_toggle\RoleToggleState
   */
  private $roleToggleState;

  /**
   * @var \Drupal\role_toggle\RoleToggleUrlManager
   */
  private $roleToggleUrlManager;

  /**
   * @var \Drupal\role_toggle\RoleExcludedService
   */
  private $roleExcludedService;

  /**
   * RoleToggleForm constructor.
   *
   * @param \Drupal\role_toggle\RoleToggleState $roleToggleState
   * @param \Drupal\role_toggle\RoleToggleUrlManager $roleToggleUrlManager
   * @param \Drupal\role_toggle\RoleExcludedService $roleExcludedService
   */
  public function __construct(RoleToggleState $roleToggleState, RoleToggleUrlManager $roleToggleUrlManager, RoleExcludedService $roleExcludedService) {
    $this->roleToggleState = $roleToggleState;
    $this->roleToggleUrlManager = $roleToggleUrlManager;
    $this->roleExcludedService = $roleExcludedService;
  }


  /**
   * { @inheritDoc }
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('role_toggle.state'),
      $container->get('role_toggle.url_manager'),
      $container->get('role_toggle.role_excluded_service')
    );
  }

  /**
   * { @inheritDoc }
   */
  public function getFormId() {
    return 'role_toggle_form';
  }

  /**
   * { @inheritDoc }
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Get the arg like this to not break the interface.
    $currentRoleIds = $form_state->getBuildInfo()['args'][0];
    // This form is cacheable. See https://www.drupal.org/project/drupal/issues/2578855
//    $form['#cache']['max-age'] = Cache::PERMANENT;
//    $form['#cache']['contexts'][] = 'user.permissions';
//    $queryKey = $this->roleToggleState->getQueryKey();
//    $form['#cache']['contexts'][] = "url.query_args:$queryKey";

    $originalRolesBag = $this->roleToggleState->getOriginalRolesBag();
    foreach ($originalRolesBag->rolesWithoutLocked() as $roleId => $role) {
      $enabled = in_array($roleId, $currentRoleIds);
      $form['role_toggle'][$roleId] = array(
        '#type' => 'checkbox',
        '#default_value' => (int)$enabled,
        '#title' => $role->label(),
        '#wrapper_attributes' => [
          'class' => [$this->roleExcludedService->cssClass($role)]
        ],
      );
    }
    if(isset($form['role_toggle'])) {
      $form['role_toggle']['#tree'] = TRUE;
      $form['#attached']['library'] = [
        'role_toggle/form',
      ];
      $form['actions']['submit'] = [
        '#type' => 'submit',
        '#value' => t('Submit'),
      ];
      $form['links'] = [
        'anon' => [
          '#type' => 'link',
          '#title' => '😎',
          '#url' => $this->roleToggleUrlManager->togglerUrl(RolesBag::anonymous()),
        ],
        'none' => [
          '#type' => 'link',
          '#title' => '🡇',
          '#url' => $this->roleToggleUrlManager->togglerUrl($this->roleToggleUrlManager->defaultRolesBag()),
        ],
        'all' => [
          '#type' => 'link',
          '#title' => '🡅',
          '#url' => $this->roleToggleUrlManager->togglerUrl($originalRolesBag),
        ],
      ];
    }
    return $form;
  }

  /**
   * { @inheritDoc }
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {}

  /**
   * { @inheritDoc }
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $roleValues = $form_state->getValue('role_toggle');
    $enabledRoleIds = array_keys(array_filter($roleValues));

    $url = $this->roleToggleUrlManager->togglerUrl(RolesBag::fromAuthenticatedRoleIds($enabledRoleIds), 'redirect');
    $form_state->setRedirectUrl($url);
  }

}
