<?php

namespace Drupal\role_toggle;

use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;
use Drupal\user\Entity\Role;
use Symfony\Component\HttpFoundation\Request;

class RoleToggleUrlManager {

  const QUERY_KEY = 'roles';

  const TOGGLE_REGULAR_PERMISSION = 'role_toggle regular';

  const TOGGLE_ALL_PERMISSION = 'role_toggle all';

  const YES = '1';

  const NO = '0';

  /**
   * The role toggle state.
   *
   * @var \Drupal\role_toggle\RoleToggleState
   */
  private $roleToggleState;

  /**
   * @var \Drupal\role_toggle\RoleExcludedService
   */
  private $roleExcludedService;

  /**
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  private $currentUser;

  /**
   * RoleToggleUrlManager constructor.
   *
   * @param \Drupal\role_toggle\RoleToggleState $roleToggleState
   * @param \Drupal\role_toggle\RoleExcludedService $roleExcludedService
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   */
  public function __construct(RoleToggleState $roleToggleState, RoleExcludedService $roleExcludedService, AccountProxyInterface $currentUser) {
    $this->roleToggleState = $roleToggleState;
    $this->roleExcludedService = $roleExcludedService;
    $this->currentUser = $currentUser;
  }

  /**
   * Get query key/value array.
   *
   * @todo Explain type parameter and convert to class constants.
   *
   * @param \Drupal\role_toggle\RolesBag $rolesBag
   *
   * @param string $type
   *
   * @return array
   */
  public function getQuery(RolesBag $rolesBag, $type = 'link') {
    if ($rolesBag->equals($this->defaultRolesBag())) {
      // @see \Drupal\role_toggle\PathProcessor\RoleTogglePathProcessor::processOutbound
      $queryValue = "empty-$type";
    }
    elseif ($rolesBag->hasAuthenticated()) {
      $roleIds = $rolesBag->idsWithoutLocked();
      $toggleableRolesBag = $this->toggleableRolesBag();
      $chars = [];
      foreach ($toggleableRolesBag->idsWithoutLocked() as $toggleableRoleId) {
        $active = in_array($toggleableRoleId, $roleIds);
        $chars[] = $active ? self::YES : self::NO;
      }
      $queryValue = implode($chars);
    }
    else {
      $queryValue = '';
    }
    return [$this->queryKey() => $queryValue];
  }

  /**
   * Create current url with role toggle query.
   *
   * @param \Drupal\role_toggle\RolesBag $roleBag
   *
   * @param string $type
   *
   * @return \Drupal\Core\Url
   */
  public function togglerUrl(RolesBag $roleBag, $type = 'link') {
    $request = \Drupal::request();
    // Use this, as <current> breaks if no access.
    $url = Url::createFromRequest($request);
    $query = $request->query->all();
    unset($query[$this->queryKey()]);
    $query = $this->getQuery($roleBag, $type) + $query;
    $url->setOption('query', $query);
    return $url;
  }

  public function getCacheContexts() {
    $queryKey = $this->queryKey();
    return ["url.query_args:$queryKey"];
  }

  /**
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request, to get query key.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The account. This is called from our authentication manager decorator,
   *   before currentUser even knows about it.
   *
   * @return \Drupal\role_toggle\RolesBag|null
   */
  public function parseToggledRolesBag(Request $request, AccountInterface &$account) {
    $toggleableRolesBag = $this->toggleableRolesBag($account);
    $queryKey = $this->queryKey();
    if ($toggleableRolesBag->idsWithoutLocked() && $request->query->has($queryKey)) {
      $queryValue = $request->query->get($queryKey);
      if ($queryValue === '') {
        $rolesBag = RolesBag::anonymous();
      }
      else {
        $rolesBag = $this->parseRoles($queryValue, $toggleableRolesBag);
      }
      return $rolesBag;
    }
    return $this->defaultRolesBag();
  }

  /**
   * @param $queryValue
   * @param \Drupal\role_toggle\RolesBag $toggleableRolesBag
   *
   * @return \Drupal\role_toggle\RolesBag|null
   */
  private function parseRoles($queryValue, RolesBag $toggleableRolesBag) {
    if (strlen($queryValue) === count($toggleableRolesBag->idsWithoutLocked())) {
      $roleIds = [AccountInterface::AUTHENTICATED_ROLE];
      foreach ($toggleableRolesBag->idsWithoutLocked() as $i => $roleId) {
        $currentChar = substr($queryValue, $i, 1);
        if ($currentChar === static::YES) {
          $roleIds[] = $roleId;
        }
        elseif ($currentChar !== static::NO) {
          $roleIds = NULL;
          break;
        }
      }
    }
    return isset($roleIds) ?
      RolesBag::fromAuthenticatedRoleIds($roleIds) :
      $this->defaultRolesBag();
  }

  /**
   * @param \Drupal\Core\Session\AccountInterface $account
   *   If this is called from the authenticator, we need the to-be authenticated
   *   user, which is not yet set in currentUser.
   *
   * @return \Drupal\role_toggle\RolesBag
   */
  public function toggleableRolesBag(AccountInterface $account = NULL) {
    if (!$account) {
      $account = $this->currentUser;
    }
    if ($this->accountMayToggleAllRoles($account)) {
      return $this->roleToggleState->getOriginalRolesBag();
    }
    elseif ($this->accountMayToggleRegularRoles($account)) {
      return $this->roleToggleState->getOriginalRolesBag()
        ->filterBy([$this->roleExcludedService, 'isIncludedRole']);
    }
    else {
      return RolesBag::fromAuthenticatedRoleIds([]);
    }
  }

  /**
   * @return \Drupal\role_toggle\RolesBag
   */
  public function defaultRolesBag() {
    return $this->roleToggleState->getOriginalRolesBag()
      ->filterBy([$this->roleExcludedService, 'isExcludedRole']);
  }

  /**
   * @param \Drupal\Core\Session\AccountInterface $account
   *
   * @return bool
   */
  private function accountMayToggleRegularRoles(AccountInterface &$account) {
    return $account->hasPermission(self::TOGGLE_REGULAR_PERMISSION);
  }

  /**
   * @param \Drupal\Core\Session\AccountInterface $account
   *
   * @return bool
   */
  private function accountMayToggleAllRoles(AccountInterface &$account) {
    return $account->hasPermission(self::TOGGLE_ALL_PERMISSION);
  }

  /**
   * @return string
   */
  public function queryKey() {
    return self::QUERY_KEY;
  }

}
