<?php

namespace Drupal\role_toggle;

use Drupal\Core\Session\AccountInterface;
use Drupal\user\Entity\Role;
use Drupal\user\RoleInterface;

class RolesBag {

  /**
   * @var RoleInterface[]
   */
  private $roles;

  /**
   * RolesBag constructor.
   *
   * @param \Drupal\user\RoleInterface[] $roles
   */
  private function __construct(array $roles) {
    $roleIds = array_map(function(RoleInterface $role) {
      return $role->id();
    }, $roles);
    $this->roles = array_combine($roleIds, $roles);
    uasort($this->roles, [Role::class, 'sort']);
  }

  private static function fromRoleIds(array $ids) {
    $roles = Role::loadMultiple($ids);
    return new static($roles);
  }

  /**
   * @return static
   */
  public static function anonymous() {
    return static::fromRoleIds([RoleInterface::ANONYMOUS_ID]);
  }

  /**
   * @param string[] $ids
   *
   * @return static
   */
  public static function fromAuthenticatedRoleIds($ids) {
    if (!in_array(RoleInterface::AUTHENTICATED_ID, $ids)) {
      $ids[] = RoleInterface::AUTHENTICATED_ID;
    }
    return static::fromRoleIds($ids);
  }

  /**
   * @return static
   */
  public static function all() {
    $roles = Role::loadMultiple();
    return static::fromAuthenticatedRoleIds(array_keys(array_diff_key($roles, array_flip(self::lockedRoleIds()))));
  }

  /**
   * @param \Drupal\Core\Session\AccountInterface $account
   *
   * @return static
   */
  public static function fromAccount(AccountInterface $account) {
    return static::fromRoleIds($account->getRoles());
  }

  /**
   * @return string[]
   */
  private static function lockedRoleIds() {
    return [RoleInterface::ANONYMOUS_ID, RoleInterface::AUTHENTICATED_ID];
  }

  /**
   * @param \Drupal\role_toggle\RolesBag $rolesBag
   *
   * @return bool
   */
  public function equals(RolesBag $rolesBag) {
    return $this->idsWithLocked() === $rolesBag->idsWithLocked();
  }

  /**
   * @param callable $callable
   *
   * @return static
   */
  public function filterBy(Callable $callable) {
    if ($this->hasAuthenticated()) {
      $filteredRoles = array_filter($this->rolesWithoutLocked(), $callable);
      return static::fromAuthenticatedRoleIds(array_keys($filteredRoles));
    }
    else {
      return $this;
    }
  }

  /**
   * @return bool
   */
  public function hasAnonymous() {
    return in_array(RoleInterface::ANONYMOUS_ID, $this->idsWithLocked());
  }

  /**
   * @return bool
   */
  public function hasAuthenticated() {
    return in_array(RoleInterface::AUTHENTICATED_ID, $this->idsWithLocked());
  }

  /**
   * @return string[]
   */
  public function idsWithLocked() {
    return array_keys($this->roles);
  }

  /**
   * @return \Drupal\user\RoleInterface[]
   */
  public function rolesWithLocked() {
    return $this->roles;
  }
  
  /**
   * @return string[]
   */
  public function idsWithoutLocked() {
    return array_values(array_diff($this->idsWithLocked(), self::lockedRoleIds()));
  }

  /**
   * @return \Drupal\user\RoleInterface[]
   */
  public function rolesWithoutLocked() {
    return array_diff_key($this->rolesWithLocked(), array_flip(self::lockedRoleIds()));
  }

}
